import 'dart:async';

import 'package:dio/dio.dart';

typedef ThyiInterceptor = void Function(ApiMethod original);

class Thyi {
  Dio dio;
  List<ThyiInterceptor> interceptors;

  Thyi(String baseUrl, {this.interceptors}) {
    this.interceptors = this.interceptors?? [];
    dio = Dio(BaseOptions(baseUrl: baseUrl));
  }

}

/// 代表某条api
class ApiMethod {
  String method;
  String path = '';
  Map<String, String> headers = {};
  Map<String, String> fields = {};
  Map<String, String> queries = {};

  ApiMethod(this.method, this.path, {this.headers, this.fields, this.queries});

  Future<String> send(Thyi thyi) {
    if (thyi.interceptors != null && thyi.interceptors.isNotEmpty) {
      thyi.interceptors.forEach((i) => i(this));
    }

    var options = Options();
    var dio = thyi.dio;
    
    if (headers != null && headers.isNotEmpty) {
      options.headers.addAll(headers);
    }

    if (isGet()) {
      print('url: ${dio.options.baseUrl}, path: ${path}');
      return dio.get<String>(path, queryParameters: queries, options: options)
        .then((response) => response.data.toString());
    }
    else if (isPost()) {
      return dio.post<String>(path, queryParameters: queries, options: options, data: fields)
        .then((response) => response.data.toString());
    } else {
      return Future.error('for now unsupport http method; ${method}');
    }
  }

  bool isGet() {
    return "GET" == method.toUpperCase();
  }

  bool isPost() {
    return "POST" == method.toUpperCase();
  }

}